@@ -1,5 +1,18 @@
 //! Shared code used in both the linker and no-linker implementations of this crate.
+
 // Copyright 2021 Oxide Computer Company
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
 
 use crate::{DataType, Provider};
 use proc_macro2::TokenStream;
@@ -86,6 +99,7 @@ pub fn generate_type_check(
     quote! {
         let __usdt_private_args_lambda = $args_lambda;
         #[allow(unused_imports)]
+        #[allow(non_snake_case)]
         #(#use_statements)*
         fn #type_check_function (#(#type_check_args),*) { }
         let _ = || {
@@ -209,7 +223,6 @@ pub(crate) fn build_probe_macro(
     provider: &Provider,
     probe_name: &str,
     types: &[DataType],
-    pre_macro_block: TokenStream,
     impl_block: TokenStream,
 ) -> TokenStream {
     let module = config.module_ident();
@@ -222,7 +235,6 @@ pub(crate) fn build_probe_macro(
         quote! {}
     };
     quote! {
-        #pre_macro_block
         #[allow(unused_macros)]
         macro_rules! #macro_name {
             #no_args_match
@@ -270,6 +282,7 @@ mod tests {
         let expected = quote! {
             let __usdt_private_args_lambda = $args_lambda;
             #[allow(unused_imports)]
+            #[allow(non_snake_case)]
             fn __usdt_private_provider_probe_type_check(
                 _: impl ::std::borrow::Borrow<u8>,
                 _: impl ::std::borrow::Borrow<i64>
@@ -292,6 +305,7 @@ mod tests {
         let expected = quote! {
             let __usdt_private_args_lambda = $args_lambda;
             #[allow(unused_imports)]
+            #[allow(non_snake_case)]
             fn __usdt_private_provider_probe_type_check(_: impl AsRef<str>) { }
             let _ = || {
                 let args = (__usdt_private_args_lambda.clone()(),);
@@ -311,6 +325,7 @@ mod tests {
         let expected = quote! {
             let __usdt_private_args_lambda = $args_lambda;
             #[allow(unused_imports)]
+            #[allow(non_snake_case)]
             fn __usdt_private_provider_probe_type_check(_: impl AsRef<[u8]>) { }
             let _ = || {
                 let args = (__usdt_private_args_lambda.clone()(),);
@@ -330,6 +345,7 @@ mod tests {
         let expected = quote! {
             let __usdt_private_args_lambda = $args_lambda;
             #[allow(unused_imports)]
+            #[allow(non_snake_case)]
             use my_module::MyType;
             fn __usdt_private_provider_probe_type_check(_: impl ::std::borrow::Borrow<MyType>) { }
             let _ = || {
