@@ -1,6 +1,18 @@
 //! Implementation of USDT functionality on platforms without runtime linker support.
 
 // Copyright 2021 Oxide Computer Company
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
 
 use crate::record::{emit_probe_record, process_section};
 use crate::{common, Probe, Provider};
@@ -53,7 +65,7 @@ fn compile_provider(provider: &Provider, config: &crate::CompileProvidersConfig)
         .collect::<Vec<_>>();
     let module = config.module_ident();
     quote! {
-        mod #module {
+        pub(crate) mod #module {
             #(#probe_impls)*
         }
     }
@@ -67,7 +79,6 @@ fn compile_probe(
     let (unpacked_args, in_regs) = common::construct_probe_args(&probe.types);
     let is_enabled_rec = emit_probe_record(&provider.name, &probe.name, None);
     let probe_rec = emit_probe_record(&provider.name, &probe.name, Some(&probe.types));
-    let pre_macro_block = TokenStream::new();
     let impl_block = quote! {
         {
             let mut is_enabled: u64;
@@ -93,14 +104,7 @@ fn compile_probe(
             }
         }
     };
-    common::build_probe_macro(
-        config,
-        provider,
-        &probe.name,
-        &probe.types,
-        pre_macro_block,
-        impl_block,
-    )
+    common::build_probe_macro(config, provider, &probe.name, &probe.types, impl_block)
 }
 
 fn extract_probe_records_from_section() -> Result<Option<Section>, crate::Error> {
