@@ -2,6 +2,18 @@
 //! probe information in an object file.
 
 // Copyright 2021 Oxide Computer Company
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
 
 use std::{
     collections::BTreeMap,
@@ -324,8 +336,8 @@ pub(crate) fn emit_probe_record(prov: &str, probe: &str, types: Option<&[DataTyp
         version = PROBE_REC_VERSION,
         n_args = n_args,
         flags = if is_enabled { 1 } else { 0 },
-        prov = prov,
-        probe = probe,
+        prov = prov.replace("__", "-"),
+        probe = probe.replace("__", "-"),
         arguments = arguments,
         yeet = if cfg!(target_os = "illumos") {
             // The illumos linker may yeet our probes section into the trash under
@@ -540,4 +552,19 @@ mod test {
                 .is_some());
         }
     }
+
+    #[test]
+    fn test_emit_probe_record_dunders() {
+        let provider = "provider";
+        let probe = "my__probe";
+        let types = [
+            DataType::Native(dtrace_parser::DataType::U8),
+            DataType::Native(dtrace_parser::DataType::String),
+        ];
+        let record = emit_probe_record(provider, probe, Some(&types));
+        assert!(
+            record.contains("my-probe"),
+            "Expected double-underscores to be translated to a single dash"
+        );
+    }
 }
